import { isLoadedMarkdownPage } from "@vitebook/core";
import { tick } from "svelte";
import { get } from "svelte/store";
import { configureRouter } from ":virtual/vitebook/app";
import DefaultNotFound from "./components/DefaultNotFound.svelte";
import { createMemoryHistory } from "./router/history/memory";
import { Router } from "./router/router";
import { layouts as pageLayouts } from "./stores/layouts";
import { page } from "./stores/page";
import { pages } from "./stores/pages";
async function createRouter() {
  const router = new Router({
    baseUrl: "/",
    history: import.meta.env.SSR ? createMemoryHistory() : window.history
  });
  addRoutes(router, get(pages));
  handleHMR(router);
  await configureRouter?.(router);
  if (!import.meta.env.SSR) {
    router.initListeners();
    await router.go(location.href, { replace: true });
  }
  return router;
}
let routes = [];
function addRoutes(router, pages2) {
  pages2.forEach((_page) => {
    router.addRoute({
      path: _page.route,
      loader: () => loadPage(_page),
      prefetch: async () => {
        await loadPage(_page, { prefetch: true });
      }
    });
    routes.push(_page.route);
    if (import.meta.hot && get(page)?.route === decodeURI(_page.route)) {
      loadPage(_page);
    }
  });
  if (!router.hasRoute("/404.html")) {
    router.addRoute({
      path: "/404.html",
      loader: async () => DefaultNotFound
    });
  }
}
function handleHMR(router) {
  if (import.meta.hot) {
    const updatePages = (pages2) => {
      routes.forEach((route) => router.removeRoute(route));
      routes = [];
      addRoutes(router, pages2);
    };
    let pagesUpdateTimer;
    const delayedPagesUpdate = (pages2) => {
      clearTimeout(pagesUpdateTimer);
      pagesUpdateTimer = setTimeout(() => updatePages(pages2), 100);
    };
    pages.subscribe(($pages) => delayedPagesUpdate($pages));
    pageLayouts.subscribe(() => delayedPagesUpdate(get(pages)));
  }
}
async function loadPage(_page, { prefetch = false } = {}) {
  const mod = await _page.loader();
  const component = mod.default;
  const $pageLayouts = get(pageLayouts);
  const layouts = await Promise.all(_page.layouts.map((i) => $pageLayouts[i]).map(async (layout) => {
    const mod2 = await layout.loader();
    return {
      $$loaded: true,
      ...layout,
      module: mod2,
      get component() {
        return mod2.default;
      }
    };
  }));
  if (!prefetch) {
    page.__set({
      ..._page,
      $$loaded: true,
      module: mod,
      layouts,
      get component() {
        return mod.default;
      },
      get meta() {
        return isLoadedMarkdownPage(this) ? mod.meta : void 0;
      }
    });
  }
  await tick();
  return component;
}
export {
  createRouter
};
