import { ensureLeadingSlash, inBrowser, isString } from "@vitebook/core";
import { tick } from "svelte";
import { noop, raf } from "svelte/internal";
import { get } from "svelte/store";
import { pages } from "../stores/pages";
import { route } from "../stores/route";
class Router {
  constructor({ baseUrl, history, routes }) {
    this.routes = /* @__PURE__ */ new Map();
    this.enabled = true;
    this.baseUrl = "/";
    this.scrollBehaviour = "auto";
    this.scrollOffset = () => ({ top: 0, left: 0 });
    this._isReady = false;
    this._isReadyPromise = new Promise((res) => this._resolveReadyPromise = () => {
      res(void 0);
      this._isReady = true;
    });
    this.baseUrl = baseUrl;
    this.history = history;
    routes?.forEach((route2) => {
      this.addRoute(route2);
    });
    if (inBrowser) {
      document.body.setAttribute("tabindex", "-1");
      this.history.replaceState(this.history.state || {}, "", location.href);
    }
  }
  get isReady() {
    return this._isReady;
  }
  get waitUntilReady() {
    return this._isReadyPromise;
  }
  hasRoute(path) {
    return this.routes.has(decodeURI(path));
  }
  addRoute(route2) {
    this.routes.set(decodeURI(route2.path), route2);
  }
  removeRoute(route2) {
    this.routes.delete(decodeURI(isString(route2) ? route2 : route2.path));
  }
  owns(url) {
    if (import.meta.env.SSR)
      return true;
    return url.origin === location.origin && url.pathname.startsWith(this.baseUrl);
  }
  findRoute(url) {
    if (this.owns(url)) {
      const path = ensureLeadingSlash(url.pathname.slice(this.baseUrl.length) || "/");
      const decodedPath = decodeURI(path);
      const route2 = this.routes.get(decodedPath) ?? this.routes.get("/404.html");
      const query = new URLSearchParams(url.search);
      const id = `${path}?${query}`;
      return { ...route2, id, url };
    }
    return void 0;
  }
  back() {
    this.history.back();
  }
  async go(href, {
    scroll = void 0,
    replace = false,
    keepfocus = false,
    state = {}
  } = {}) {
    const url = new URL(href, href.startsWith("#") ? /(.*?)(#|$)/.exec(location.href)[1] : getBaseUri(this.baseUrl));
    await this.beforeNavigate?.(url);
    if (this.enabled && this.owns(url)) {
      this.history[replace ? "replaceState" : "pushState"](state, "", href);
      await this.navigate({
        url,
        scroll,
        keepfocus,
        hash: url.hash
      });
      if (this._resolveReadyPromise) {
        this._resolveReadyPromise();
        this._resolveReadyPromise = void 0;
      }
      return;
    }
    location.href = url.href;
    return new Promise(() => {
    });
  }
  async prefetch(url) {
    const routeLocation = this.findRoute(url);
    if (routeLocation?.redirect) {
      await this.prefetch(new URL(routeLocation.redirect, getBaseUri(this.baseUrl)));
      return;
    }
    if (!routeLocation) {
      throw new Error("Attempted to prefetch a URL that does not belong to this app");
    }
    await routeLocation.prefetch?.(routeLocation);
  }
  async navigate({
    url,
    keepfocus,
    scroll,
    hash
  }) {
    const routeLocation = this.findRoute(url);
    if (routeLocation?.redirect) {
      await this.go(routeLocation.redirect, { replace: true });
      return;
    }
    if (!routeLocation) {
      throw new Error("Attempted to navigate to a URL that does not belong to this app");
    }
    const component = await routeLocation.loader(routeLocation);
    if (!get(pages).find((page) => page.route === routeLocation.url.pathname)) {
      await this.go("/404.html");
      return;
    }
    route.__set({
      ...routeLocation,
      component: component.default ?? component
    });
    await tick();
    if (inBrowser) {
      if (!keepfocus) {
        document.body.focus();
      }
      if (scroll !== false) {
        await this.scrollToPosition({ scroll, hash });
      }
    }
    await this.afterNavigate?.(url);
  }
  async scrollToPosition({
    scroll,
    hash
  }) {
    if (!inBrowser)
      return;
    const deepLinked = hash && document.getElementById(decodeURI(hash).slice(1));
    const scrollTo = (options) => {
      if ("scrollBehavior" in document.documentElement.style) {
        window.scrollTo(options);
      } else {
        window.scrollTo(options.left ?? 0, options.top ?? 0);
      }
    };
    await raf(noop);
    if (scroll) {
      scrollTo({
        left: scroll.x,
        top: scroll.y,
        behavior: this.scrollBehaviour
      });
    } else if (deepLinked) {
      const docRect = document.documentElement.getBoundingClientRect();
      const elRect = deepLinked.getBoundingClientRect();
      const offset = this.scrollOffset();
      const left = elRect.left - docRect.left - offset.left;
      const top = elRect.top - docRect.top - offset.top;
      scrollTo({ left, top, behavior: this.scrollBehaviour });
    } else {
      scrollTo({ left: 0, top: 0, behavior: this.scrollBehaviour });
    }
  }
  initListeners() {
    if ("scrollRestoration" in this.history) {
      this.history.scrollRestoration = "manual";
    }
    addEventListener("beforeunload", () => {
      this.history.scrollRestoration = "auto";
    });
    addEventListener("load", () => {
      this.history.scrollRestoration = "manual";
    });
    let scrollTimer;
    addEventListener("scroll", () => {
      clearTimeout(scrollTimer);
      scrollTimer = setTimeout(() => {
        const newState = {
          ...this.history.state || {},
          "vitebook:scroll": scrollState()
        };
        this.history.replaceState(newState, document.title, window.location.href);
      }, 50);
    });
    const hasPrefetched = /* @__PURE__ */ new Set();
    const triggerPrefetch = (event) => {
      const a = findAnchor(event.target);
      if (!a || !a.href || hasPrefetched.has(getHref(a)) || !getHref(a).startsWith(getBaseUri(this.baseUrl)) || !this.routes.has(decodeURI(getHrefURL(a).pathname))) {
        return;
      }
      this.prefetch(getHrefURL(a));
      hasPrefetched.add(getHref(a));
    };
    let mouseMoveTimeout;
    const handleMouseMove = (event) => {
      clearTimeout(mouseMoveTimeout);
      mouseMoveTimeout = setTimeout(() => {
        triggerPrefetch(event);
      }, 20);
    };
    addEventListener("touchstart", triggerPrefetch);
    addEventListener("mousemove", handleMouseMove);
    addEventListener("click", (event) => {
      if (!this.enabled)
        return;
      if (event.button || event.which !== 1)
        return;
      if (event.metaKey || event.ctrlKey || event.shiftKey || event.altKey)
        return;
      if (event.defaultPrevented)
        return;
      const a = findAnchor(event.target);
      if (!a || !a.href)
        return;
      const url = getHrefURL(a);
      const urlString = url.toString();
      if (urlString === location.href) {
        event.preventDefault();
        if (location.hash) {
          this.scrollToPosition({ hash: location.hash });
        }
        return;
      }
      const rel = (a.getAttribute("rel") || "").split(/\s+/);
      if (a.hasAttribute("download") || rel && rel.includes("external")) {
        return;
      }
      if (a instanceof SVGAElement ? a.target.baseVal : a.target)
        return;
      if (!this.owns(url))
        return;
      const i1 = urlString.indexOf("#");
      const i2 = location.href.indexOf("#");
      const u1 = i1 >= 0 ? urlString.substring(0, i1) : urlString;
      const u2 = i2 >= 0 ? location.href.substring(0, i2) : location.href;
      if (u1 === u2) {
        this.go(decodeURI(url.hash), { replace: true });
        return;
      }
      this.go(url.href);
      event.preventDefault();
    });
    addEventListener("popstate", (event) => {
      if (event.state && this.enabled) {
        const url = new URL(location.href);
        this.navigate({
          url,
          scroll: event.state["vitebook:scroll"]
        });
      }
    });
  }
}
function scrollState() {
  return {
    x: scrollX,
    y: scrollY
  };
}
function findAnchor(node) {
  while (node && node.nodeName.toUpperCase() !== "A")
    node = node.parentNode;
  return node;
}
function getHref(node) {
  return node instanceof SVGAElement ? node.href.baseVal : node.href;
}
function getHrefURL(node) {
  return node instanceof SVGAElement ? new URL(node.href.baseVal, document.baseURI) : new URL(node.href);
}
function getBaseUri(baseUrl = "/") {
  return import.meta.env.SSR ? `https://ssr.com/${baseUrl}` : `${location.protocol}//${location.host}${baseUrl === "/" ? "" : baseUrl}`;
}
export {
  Router
};
